# Copyright (c) 2015 Ultimaker B.V.
# Uranium is released under the terms of the LGPLv3 or higher.

from UM.Scene.ToolHandle import ToolHandle
from UM.Mesh.MeshBuilder import MeshBuilder
from UM.Math.Vector import Vector
from UM.Math.Color import Color
from UM.Logger import Logger
import math
from enum import IntEnum
from UM.Mesh.MeshData import MeshData

from typing import Optional, cast
from UM.Qt.Bindings.Theme import Theme
from UM.Qt.QtApplication import QtApplication
##
##  NOT ANY LONGER IN USE
##

class LeftCornerToolHandle(ToolHandle):

    def __init__(self, parent = None):
        super().__init__(parent)
        self._auto_scale = False
        self._cornerHandleSize = 0.5
        self._spawnRectSize = 2.5
        self._thickness = 0.5

    def _onSelectionCenterChanged(self) -> None:
        return None

    def getExtraWidgetsColorMap(self):
        return self._extra_widgets_color_map

    def buildMesh(self):
        #SOLIDMESH
        mb = MeshBuilder()

        mb.addCube(
            width = self._thickness,
            height = self._cornerHandleSize,
            depth = self._cornerHandleSize,
            center = Vector(0,(self._spawnRectSize/2.0),-(self._spawnRectSize/2.0)),
            color = Color(1.0, 1.0, 1.0, 1.0)#self._extra_widgets_color_map[self.ExtraWidgets.RectHandle.value]
        )

        self.setSolidMesh(mb.build())

        mb = MeshBuilder()


        mb.addCube(
            width = self._thickness,
            height = self._cornerHandleSize,
            depth = self._cornerHandleSize,
            center = Vector(0,(self._spawnRectSize/2.0),-(self._spawnRectSize/2.0)),
            color = ToolHandle.YAxisSelectionColor
        )



        self.setSelectionMesh(mb.build())

class RightCornerToolHandle(ToolHandle):

    def __init__(self, parent = None):
        super().__init__(parent)
        self._auto_scale = False
        self._cornerHandleSize = 0.5
        self._spawnRectSize = 2.5
        self._thickness = 0.5

    def _onSelectionCenterChanged(self) -> None:
        return None

    def getExtraWidgetsColorMap(self):
        return self._extra_widgets_color_map

    def buildMesh(self):
        #SOLIDMESH
        mb = MeshBuilder()

        mb.addCube(
            width = self._thickness,
            height = self._cornerHandleSize,
            depth = self._cornerHandleSize,
            center = Vector(0,-(self._spawnRectSize/2.0),(self._spawnRectSize/2.0)),
            color = Color(1.0, 1.0, 1.0, 1.0)
        )

        self.setSolidMesh(mb.build())

        mb = MeshBuilder()


        mb.addCube(
            width = self._thickness,
            height = self._cornerHandleSize,
            depth = self._cornerHandleSize,
            center = Vector(0,-(self._spawnRectSize/2.0),(self._spawnRectSize/2.0)),
            color = ToolHandle.ZAxisSelectionColor
        )



        self.setSelectionMesh(mb.build())

SPAWN_RECT_SIZE = 2.5
THICKNESS = 0.5
CORNER_HANDLE_SIZE = 0.5

class AutoparterToolHandle(ToolHandle):



    @classmethod
    def getCubicMesh(cls, female) -> MeshData:
        mb = MeshBuilder()

        theme = cast(Theme, QtApplication.getInstance().getTheme())

        c = theme.getColor("primary").getRgb()
        c_md = theme.getColor("model_default").getRgb()
        col = Color(c[0]/255.0, c[1]/255.0, c[2]/255.0, c[3]/255.0)
        colDark = Color(c[0]/255.0*0.5, c[1]/255.0*0.5, c[2]/255.0*0.5, c[3]/255.0)
        colDef = Color(c_md[0]/255.0, c_md[1]/255.0, c_md[2]/255.0, c_md[3]/255.0)

        if female:
            col = Color(0.7, 0.0, 0.0, 1.0)
            colDef = Color(1.0, 0.0, 0.0, 1.0)
        else:
            col = Color(0.0, 0.7, 0.0, 1.0)
            colDef = Color(0.0, 1.0, 0.0, 1.0)


        mb.addCube(
            width = THICKNESS,
            height = SPAWN_RECT_SIZE,
            depth = SPAWN_RECT_SIZE,
            center = Vector(0,0,0),
            color = col
        )

        return mb.build()

    def changeHandleMode(self, female=True, disabled=False):
        #disabled = False
        if female == self._isFemale and self._isDisabled == disabled:
            return
        self._isDisabled = disabled
        self._isFemale = female
        mesh = self._femaleMesh if female else self._maleMesh
        if disabled:
            mesh = self._disabledMesh
        self.setSolidMesh(mesh)

    def __init__(self, parent = None):
        super().__init__(parent)
        self._femaleMesh = None
        self._maleMesh = None

        self._disabledMesh = None
        self._isDisabled = False

        self._isFemale = True

        self._auto_scale = False
        self._cornerHandleSize = 0.5
        self._spawnRectSize = 2.5
        self._thickness = 0.5

    def _onSelectionCenterChanged(self) -> None:
        return None

    def getExtraWidgetsColorMap(self):
        return self._extra_widgets_color_map


    def buildMesh(self):
        #SOLIDMESH
        mb = MeshBuilder()
        mb.addCube(
            width = self._thickness,
            height = self._spawnRectSize,
            depth = self._spawnRectSize,
            center = Vector(0,0,0),
            color = Color(1.0, 0.0, 0.0, 1.0)
        )
        self._femaleMesh = mb.build()

        mb = MeshBuilder()
        mb.addCube(
            width = self._thickness,
            height = self._spawnRectSize,
            depth = self._spawnRectSize,
            center = Vector(0,0,0),
            color = Color(0.0, 1.0, 0.0, 1.0)
        )
        self._maleMesh = mb.build()

        mb = MeshBuilder()
        mb.addCube(
            width = self._thickness,
            height = self._spawnRectSize,
            depth = self._spawnRectSize,
            center = Vector(0,0,0),
            color = Color(0.0, 0.0, 0.0, 1.0)
        )
        self._disabledMesh = mb.build()

        self.setSolidMesh(self._femaleMesh)

        mb = MeshBuilder()

        mb.addCube(
            width = 0.5,
            height = 2.5,
            depth = 2.5,
            center = Vector(0,0,0),
            color = ToolHandle.XAxisSelectionColor
        )

        self.setSelectionMesh(mb.build())

class InnerRectToolHandle(ToolHandle):

    def __init__(self, parent = None):
        super().__init__(parent)
        self._femaleMesh = None
        self._maleMesh = None

        self._disabledMesh = None
        self._isDisabled = False

        self._isFemale = True

        self._auto_scale = False
        self._cornerHandleSize = 0.5
        self._spawnRectSize = 2.5
        self._thickness = 0.5
        self._scaleF = 1.0



    def changeHandleMode(self, female=True, disabled=False):
        if female == self._isFemale and self._isDisabled == disabled:
            return
        self._isDisabled = disabled
        self._isFemale = female
        mesh = self._femaleMesh if female else self._maleMesh
        if disabled:
            mesh = self._disabledMesh
        self.setSolidMesh(mesh)

    def _onSelectionCenterChanged(self) -> None:
        return None

    def getExtraWidgetsColorMap(self):
        return self._extra_widgets_color_map

    def buildMesh(self):
        #SOLIDMESH
        mb = MeshBuilder()

        mb.addCube(
            width = self._thickness,
            height = self._spawnRectSize*self._scaleF,
            depth = self._spawnRectSize*self._scaleF,
            center = Vector(0,0,0),
            color = Color(0.7, 0.0, 0.0, 1.0)
        )
        self._femaleMesh = mb.build()

        mb = MeshBuilder()

        mb.addCube(
            width = self._thickness,
            height = self._spawnRectSize*self._scaleF,
            depth = self._spawnRectSize*self._scaleF,
            center = Vector(0,0,0),
            color = Color(0.0, 0.7, 0.0, 1.0)
        )
        self._maleMesh = mb.build()

        mb = MeshBuilder()

        mb.addCube(
            width = self._thickness,
            height = self._spawnRectSize*self._scaleF,
            depth = self._spawnRectSize*self._scaleF,
            center = Vector(0,0,0),
            color = Color(0.25, 0.25, 0.25, 1.0)
        )
        self._disabledMesh = mb.build()

        self.setSolidMesh(self._femaleMesh)

        mb = MeshBuilder()

        mb.addCube(
            width = 0.5,
            height = 2.5,
            depth = 2.5,
            center = Vector(0,0,0),
            color = ToolHandle.XAxisSelectionColor
        )

        self.setSelectionMesh(mb.build())
