# Copyright (c) 2015 Ultimaker B.V.
# Cura is released under the terms of the LGPLv3 or higher.
from typing import Optional, TYPE_CHECKING

from UM.Application import Application
from UM.Math.Polygon import Polygon
from UM.Qt.QtApplication import QtApplication
from UM.Scene.SceneNode import SceneNode
from UM.Resources import Resources
from UM.Math.Color import Color
from UM.Mesh.MeshBuilder import MeshBuilder
from UM.View.GL.OpenGL import OpenGL
from UM.Mesh.MeshData import MeshData



class PrettyNode(SceneNode):

    """ used in some cases to render a node with normal colors """

    def __init__(self, parent = None):
        super().__init__(parent)
        self._shader = None
        self._solid_mesh = None

    def getSolidMesh(self) -> Optional[MeshData]:
        return self._solid_mesh

    def setSolidMesh(self, mesh: MeshData) -> None:
        self._solid_mesh = mesh
        self.meshDataChanged.emit(self)

    def render(self, renderer):
        if not self._shader:
            self._shader = OpenGL.getInstance().createShaderProgram(Resources.getPath(Resources.Shaders, "toolhandle.shader"))
        renderer.queueNode(self, mesh = self._solid_mesh, overlay = True, shader = self._shader)
        return True

    def _calculateAABB(self) -> None:
        if self._solid_mesh:
            aabb = self._solid_mesh.getExtents(self.getWorldTransformation())
        else:  # If there is no mesh_data, use a boundingbox that encompasses the local (0,0,0)
            position = self.getWorldPosition()
            aabb = AxisAlignedBox(minimum = position, maximum = position)

        for child in self._children:
            if aabb is None:
                aabb = child.getBoundingBox()
            else:
                aabb = aabb + child.getBoundingBox()
        self._aabb = aabb
