// Copyright (c) 2019 Ultimaker B.V.
// Uranium is released under the terms of the LGPLv3 or higher.

import QtQuick 2.7
import QtQuick.Controls 2.3
import QtQuick.Layouts 1.3


import UM 1.2 as UM
import Cura 1.0 as Cura

import "sub"


Item
{
    id: base
    width: visibleChildren[0] ? visibleChildren[0].width : 0
    height: visibleChildren[0] ? visibleChildren[0].height : 0

    /******/
    /************** Window One */
    /******/

    Item {
      width: childrenRect.width
      height: childrenRect.height
      visible: UM.ActiveTool.properties.getValue("ActiveSubView") == 0
      id: windowOne

      Column
      {
          id: items2
          anchors.top: parent.top;
          anchors.left: parent.left;
          spacing: UM.Theme.getSize("default_margin").height;
          Text {
              wrapMode: Text.WordWrap
              text: "Done!"
              font: UM.Theme.getFont("medium_bold")
              color: UM.Theme.getColor("text")
          }

          Text {
              width: 200
              text: "Add connectors for easy assembly \n (If you like)"
              font: UM.Theme.getFont("default")
              color: UM.Theme.getColor("text")
              wrapMode: Text.WordWrap
          }

          Row {
            anchors.right: parent.right
            Cura.PrimaryButton
            {
                height: UM.Theme.getSize("setting_control").height;

                text: "Create Connectors"
                onClicked:
                {
                    UM.ActiveTool.triggerAction("_createConnectorsPressed");
                }
            }

          }

          Row {
            anchors.right: parent.right

            Cura.SecondaryButton
            {

                height: UM.Theme.getSize("setting_control").height;
                text: "..or complete"
                onClicked:
                {
                    UM.ActiveTool.triggerAction("_cancelPressed");
                }

            }
          }


      }

    }
    /******
    /************** Window Two
    Create Connectors
    *****/

    Item {

      width: childrenRect.width
      height: childrenRect.height
      visible: UM.ActiveTool.properties.getValue("ActiveSubView") == 1
      id: windowTwo

      Text {
          id: createConnectorsLabel
          wrapMode: Text.WordWrap
          text: "Create Connectors"
          font: UM.Theme.getFont("medium_bold")
          color: UM.Theme.getColor("text")
      }

      // (Change perspective A / B) - View
      Row {
          layoutDirection: Qt.RightToLeft
          anchors.horizontalCenter: items.horizontalCenter
          spacing: UM.Theme.getSize("thin_margin").height;
          anchors.top: parent.top;

          //binding loop :(
          anchors.right: parent.right;

          Rectangle {
            width: childrenRect.width
            height: childrenRect.height
            color: !UM.ActiveTool.properties.getValue("OneHalfSelected") ? UM.Theme.getColor("toolbar_button_active") : "transparent"
            radius:  UM.Theme.getSize("default_radius").width

            UM.SimpleButton {
              width: UM.Theme.getSize("small_button").width*0.85
              height: UM.Theme.getSize("small_button").height*0.85
              hoverColor: UM.Theme.getColor("small_button_text_hover")
              color: UM.Theme.getColor("small_button_text")
              iconMargin: UM.Theme.getSize("default_lining").width
              iconSource: "../icons/partb.svg"
              onClicked:
              {
                  UM.ActiveTool.triggerAction("_switchToCutSurfaceB");
              }
            }

        }

        Rectangle {
          width: childrenRect.width
          height: childrenRect.height
          color: UM.ActiveTool.properties.getValue("OneHalfSelected") ? UM.Theme.getColor("toolbar_button_active") : "transparent"
          radius:  UM.Theme.getSize("default_radius").width

          UM.SimpleButton {
            width: UM.Theme.getSize("small_button").width*0.85
            height: UM.Theme.getSize("small_button").height*0.85
            hoverColor: UM.Theme.getColor("small_button_text_hover")
            color: UM.Theme.getColor("small_button_text")
            iconMargin: UM.Theme.getSize("default_lining").width
            iconSource: "../icons/parta.svg"
            onClicked:
            {
                UM.ActiveTool.triggerAction("_switchToCutSurfaceA");
            }
          }
        }

      }

    //List View of Connectors

      UM.I18nCatalog { id: catalog; name: "uranium"}

      Column
      {
        id: items
        anchors.top: createConnectorsLabel.bottom;
        anchors.left: parent.left;
        anchors.topMargin: UM.Theme.getSize("default_margin").height
        spacing: UM.Theme.getSize("default_margin").height;


      Rectangle {
        radius: UM.Theme.getSize("action_button_radius").width
        width: 250
        height: Math.min(100, Math.max(configurationList.childrenRect.height,10)+12)
        border.width: 1
        border.color: UM.Theme.getColor("secondary")
        color: "transparent"

        Component {
          id: connectorsDelegate
          Rectangle {
              radius: UM.Theme.getSize("action_button_radius").width
              anchors.topMargin: UM.Theme.getSize("default_margin").height
              width: 250-12;
              height: 30
              color: "transparent"
              border.width: 1
              border.color: UM.Theme.getColor("secondary")

                RowLayout {
                  anchors.left: parent.left
                  anchors.leftMargin: UM.Theme.getSize("default_margin").height
                  height: 30
                  spacing: UM.Theme.getSize("default_margin").height;

                Text {
                    Layout.fillWidth: true
                    Layout.alignment: Qt.AlignLeft
                    text: '' + configurationList.model[index].connectorType
                    font: UM.Theme.getFont("default")
                    color: UM.Theme.getColor("text")

                }
                // connectors settings button

                  UM.SimpleButton {
                    enabled: true
                    visible: configurationList.model[index].number != "0"
                    Layout.alignment: Qt.AlignRight
                    width: UM.Theme.getSize("small_button").width*0.5
                    height: UM.Theme.getSize("small_button").height*0.5
                    hoverColor: UM.Theme.getColor("small_button_text_hover")
                    color: UM.Theme.getColor("small_button_text")
                    iconMargin: UM.Theme.getSize("default_lining").width
                    iconSource: UM.Theme.getIcon("settings")
                    onClicked: {
                      UM.ActiveTool.setProperty("SelectedConnectorIndex", index);
                      UM.ActiveTool.triggerAction("_editCubicConnector")
                    }
                  }

                  //delete connector button

                  UM.SimpleButton {
                    visible: configurationList.model[index].number != "0"
                    Layout.alignment: Qt.AlignRight
                    width: UM.Theme.getSize("small_button").width*0.5
                    height: UM.Theme.getSize("small_button").height*0.5
                    hoverColor: UM.Theme.getColor("small_button_text_hover")
                    color: UM.Theme.getColor("small_button_text")
                    iconMargin: UM.Theme.getSize("default_lining").width
                    iconSource: UM.Theme.getIcon("cross1")
                    onClicked: {
                      UM.ActiveTool.setProperty("SelectedConnectorIndex", index);
                      UM.ActiveTool.triggerAction("_deleteConnector")
                    }
                  }

            }

            }

        }

      ScrollView
      {
        id: container
        width: parent.width-10
        height:  parent.height-10
        contentHeight: configurationList.childrenRect.height
        clip: true
        anchors
            {
                top: parent.top
                left: parent.left
                leftMargin: 5
                rightMargin: 5
                topMargin: 5
                bottomMargin: 5
              }

        ListView
        {
            spacing: UM.Theme.getSize("default_margin").height/2;

            width: container.width - ((height > container.maximumHeight) ? container.ScrollBar.vertical.background.width : 0) //Make room for scroll bar if there is any.
            id: configurationList
            anchors.fill: parent
            model: UM.ActiveTool.properties.getValue("ConnectorsList")

            delegate: connectorsDelegate

        }


      }
    }

    Row {
        spacing: UM.Theme.getSize("default_margin").height;
        layoutDirection: Qt.LeftToRight
        width: parent.width

        Cura.PrimaryButton
        {

            enabled: UM.ActiveTool.properties.getValue("CanAddCubicConnector") || UM.ActiveTool.properties.getValue("CanAddShapedConnector")
            id: addButton;
            height: UM.Theme.getSize("setting_control").height;
            text: "Add"

            onClicked:
            {
                menu.open()
                //UM.ActiveTool.triggerAction("addConnector");
            }

            Menu {
              id: menu

              y: addButton.height

              background: Rectangle {
                implicitWidth: 130
                border.color: UM.Theme.getColor("border")
                color: UM.Theme.getColor("message_background")
              }

              MenuItem {
                visible: UM.ActiveTool.properties.getValue("CanAddCubicConnector")
                enabled: UM.ActiveTool.properties.getValue("CanAddCubicConnector")

                id: mitem1
                text: "Cubic..."
                onTriggered: {
                  UM.ActiveTool.triggerAction("_beginAddingCubicConnector");
                }
                background: Rectangle {
                  radius: UM.Theme.getSize("action_button_radius").width
                  //implicitWidth: mitem1.width
                  //implicitHeight: 40
                  border.color: mitem1.highlighted ? UM.Theme.getColor("primary_hover") : "transparent"
                  color: mitem1.highlighted ? UM.Theme.getColor("toolbar_button_hover") : "transparent"
                }
                contentItem: Label {
                  topPadding: UM.Theme.getSize("narrow_margin").height/2
                  bottomPadding: UM.Theme.getSize("narrow_margin").height/2
                  text: mitem1.text
                  font: UM.Theme.getFont("default")
                  color:  mitem1.highlighted ? UM.Theme.getColor("text_pressed") : UM.Theme.getColor("text")
                  verticalAlignment: Text.AlignVCenter
                }
              }
              /* //temporarily disabled
              MenuItem {
                id: mitem2
                visible: UM.ActiveTool.properties.getValue("CanAddShapedConnector")
                enabled: UM.ActiveTool.properties.getValue("CanAddShapedConnector")
                text: "Shaped..."
                background: Rectangle {
                  radius: UM.Theme.getSize("action_button_radius").width
                  //implicitWidth: mitem2.width
                  border.color: mitem2.highlighted ? UM.Theme.getColor("primary_hover") : "transparent"
                  //implicitHeight: 40
                  color: mitem2.highlighted ? UM.Theme.getColor("toolbar_button_hover") : "transparent"
                }
                contentItem: Label {
                  topPadding: UM.Theme.getSize("narrow_margin").height/2
                  bottomPadding: UM.Theme.getSize("narrow_margin").height/2
                  text: mitem2.text
                  font: UM.Theme.getFont("default")
                  color: mitem2.highlighted ? UM.Theme.getColor("text_pressed") : UM.Theme.getColor("text")
                  verticalAlignment: Text.AlignVCenter
                }
              }*/
            }

        }




  }
      // create all connectors button or cancel everything
      Row {
        width: parent.width
        layoutDirection: Qt.RightToLeft
        spacing: UM.Theme.getSize("default_margin").height;

        Cura.PrimaryButton
        {
            id: createButton;
            height: UM.Theme.getSize("setting_control").height;
            enabled: UM.ActiveTool.properties.getValue("CanCreateConnectors")
            text: "Create"
            onClicked:
            {
                UM.ActiveTool.triggerAction("_finallyCreateConnectorsPressed");
            }
        }

        Cura.SecondaryButton
        {

            id: cancelPressed2;
            height: UM.Theme.getSize("setting_control").height;
            text: "x"
            onClicked:
            {
                UM.ActiveTool.triggerAction("_cancelPressed");
            }

        }
      }



      }

  }

  /****
  Window Three
  Add a new Connector
  ****/

  Item {

    implicitWidth: 265
    implicitHeight: childrenRect.height
    visible: UM.ActiveTool.properties.getValue("ActiveSubView") == 2
    id: windowThree

    Text {
        id: newConnectorLabel
        wrapMode: Text.WordWrap
        text: "New Connector.."
        font: UM.Theme.getFont("medium_bold")
        color: UM.Theme.getColor("text")
    }

    // (Change perspective A / B) - View
    Row {
      layoutDirection: Qt.RightToLeft
      spacing: UM.Theme.getSize("thin_margin").height;
      anchors.top: parent.top;
      //binding loop :(
      anchors.right: parent.right;
      Rectangle {
        width: childrenRect.width
        height: childrenRect.height
        color: !UM.ActiveTool.properties.getValue("OneHalfSelected") ? UM.Theme.getColor("toolbar_button_active") : "transparent"
        radius:  UM.Theme.getSize("default_radius").width

        UM.SimpleButton {
          width: UM.Theme.getSize("small_button").width*0.85
          height: UM.Theme.getSize("small_button").height*0.85
          hoverColor: UM.Theme.getColor("small_button_text_hover")
          color: UM.Theme.getColor("small_button_text")
          iconMargin: UM.Theme.getSize("default_lining").width
          iconSource: "../icons/partb.svg"
          onClicked:
          {
              UM.ActiveTool.triggerAction("_switchToCutSurfaceB");
          }
        }

      }

      Rectangle {
        width: childrenRect.width
        height: childrenRect.height
        color: UM.ActiveTool.properties.getValue("OneHalfSelected") ? UM.Theme.getColor("toolbar_button_active") : "transparent"
        radius:  UM.Theme.getSize("default_radius").width

      UM.SimpleButton {
        width: UM.Theme.getSize("small_button").width*0.85
        height: UM.Theme.getSize("small_button").height*0.85
        hoverColor: UM.Theme.getColor("small_button_text_hover")
        color: UM.Theme.getColor("small_button_text")
        iconMargin: UM.Theme.getSize("default_lining").width
        iconSource: "../icons/parta.svg"
        onClicked:
        {
            UM.ActiveTool.triggerAction("_switchToCutSurfaceA");
        }
      }
    }
    }

    Rectangle
    {
     anchors.topMargin: UM.Theme.getSize("default_margin").height-1
     anchors.top: newConnectorLabel.bottom;
     anchors.left: parent.left
     id: borderTop
     width : parent.width
     height: 1
     color:  UM.Theme.getColor("tool_button_border")
    }

    Text {
        text: "View: "
        anchors.topMargin: UM.Theme.getSize("default_margin").height/2
        anchors.top: borderTop.bottom;
        anchors.left: parent.left
        wrapMode: Text.WordWrap
        font: UM.Theme.getFont("default_italic")
        color: UM.Theme.getColor("text_detail")
    }


      Row {
        id: changeViews
        anchors.top: borderTop.bottom;
        anchors.right: parent.right;
        anchors.topMargin: UM.Theme.getSize("default_margin").height/3

        layoutDirection: Qt.LeftToRight
        spacing: UM.Theme.getSize("thin_margin").height;

        Rectangle {
          width: childrenRect.width
          height: childrenRect.height
          color:  UM.ActiveTool.properties.getValue("CameraView") == 1 ? UM.Theme.getColor("toolbar_button_active") : "transparent"
          radius:  UM.Theme.getSize("default_radius").width

          UM.SimpleButton {

            width: UM.Theme.getSize("small_button").width*0.85
            height: UM.Theme.getSize("small_button").height*0.85
            hoverColor: UM.Theme.getColor("small_button_text_hover")
            color: UM.Theme.getColor("small_button_text")
            iconMargin: UM.Theme.getSize("default_lining").width
            iconSource: "../icons/view/view_front.svg"
            onClicked:
            {
                UM.ActiveTool.setProperty("CameraView", 1);
            }
          }

        }
        Rectangle {
          width: childrenRect.width
          height: childrenRect.height
          color:  UM.ActiveTool.properties.getValue("CameraView") == 2 ? UM.Theme.getColor("toolbar_button_active") : "transparent"
          radius:  UM.Theme.getSize("default_radius").width

          UM.SimpleButton {
            width: UM.Theme.getSize("small_button").width*0.85
            height: UM.Theme.getSize("small_button").height*0.85
            hoverColor: UM.Theme.getColor("small_button_text_hover")
            color: UM.Theme.getColor("small_button_text")
            iconMargin: UM.Theme.getSize("default_lining").width
            iconSource: "../icons/view/view_top.svg"
            onClicked:
            {
                UM.ActiveTool.setProperty("CameraView", 2);
            }
          }

        }
        Rectangle {
          width: childrenRect.width
          height: childrenRect.height
          color:  UM.ActiveTool.properties.getValue("CameraView") == 3 ? UM.Theme.getColor("toolbar_button_active") : "transparent"
          radius:  UM.Theme.getSize("default_radius").width

          UM.SimpleButton {
            width: UM.Theme.getSize("small_button").width*0.85
            height: UM.Theme.getSize("small_button").height*0.85
            hoverColor: UM.Theme.getColor("small_button_text_hover")
            color: UM.Theme.getColor("small_button_text")
            iconMargin: UM.Theme.getSize("default_lining").width
            iconSource: "../icons/view/view_bottom.svg"
            onClicked:
            {
                UM.ActiveTool.setProperty("CameraView", 3);
            }
          }

        }

        Rectangle {
          width: childrenRect.width
          height: childrenRect.height
          color:  UM.ActiveTool.properties.getValue("CameraView") == 4 ? UM.Theme.getColor("toolbar_button_active") : "transparent"
          radius:  UM.Theme.getSize("default_radius").width

          UM.SimpleButton {
            width: UM.Theme.getSize("small_button").width*0.85
            height: UM.Theme.getSize("small_button").height*0.85
            hoverColor: UM.Theme.getColor("small_button_text_hover")
            color: UM.Theme.getColor("small_button_text")
            iconMargin: UM.Theme.getSize("default_lining").width
            iconSource: "../icons/view/view_left.svg"
            onClicked:
            {
                  UM.ActiveTool.setProperty("CameraView", 4);
            }
          }

        }

        Rectangle {
          width: childrenRect.width
          height: childrenRect.height
          color:  UM.ActiveTool.properties.getValue("CameraView") == 5 ? UM.Theme.getColor("toolbar_button_active") : "transparent"
          radius:  UM.Theme.getSize("default_radius").width

          UM.SimpleButton {
            width: UM.Theme.getSize("small_button").width*0.85
            height: UM.Theme.getSize("small_button").height*0.85
            hoverColor: UM.Theme.getColor("small_button_text_hover")
            color: UM.Theme.getColor("small_button_text")
            iconMargin: UM.Theme.getSize("default_lining").width
            iconSource: "../icons/view/view_right.svg"
            onClicked:
            {
                UM.ActiveTool.setProperty("CameraView", 5);
            }
          }

        }

    }

    Rectangle
    {
     anchors.topMargin: UM.Theme.getSize("default_margin").height/3
     anchors.top: changeViews.bottom;
     anchors.left: parent.left
     id: borderBottom
     width : parent.width
     height: 1
     color:  UM.Theme.getColor("tool_button_border")
    }



    Column
    {
      id: newConnectorsViewC
      anchors.top: borderBottom.bottom;
      anchors.left: parent.left;
      anchors.topMargin: UM.Theme.getSize("default_margin").height
      spacing: UM.Theme.getSize("default_margin").height;


      NewConnectorView {}

    }

    Column {
      anchors.top: newConnectorsViewC.bottom;
      anchors.topMargin: UM.Theme.getSize("default_margin").height;
      spacing: UM.Theme.getSize("default_margin").height;
      anchors.right: parent.right;

      Text {
          visible: UM.ActiveTool.properties.getValue("ShowDepthError")
          id: errorLabel
          width: 200
          text: UM.ActiveTool.properties.getValue("ErrorMessage")
          wrapMode: Text.WordWrap
          font: UM.Theme.getFont("default")
          color: UM.Theme.getColor("error")

      }

    Row {

      spacing: UM.Theme.getSize("default_margin").height;

      layoutDirection: Qt.RightToLeft
      anchors.right: parent.right;

      Cura.PrimaryButton
      {
          height: UM.Theme.getSize("setting_control").height;
          enabled: UM.ActiveTool.properties.getValue("CanFinallyAddCubicConnector");
          text: !UM.ActiveTool.properties.getValue("IsEditingCubicConnector") ? "Add" : "Edit"
          onClicked:
          {
              UM.ActiveTool.triggerAction("_addCubicConnector");
          }
      }

      Cura.SecondaryButton
      {
          height: UM.Theme.getSize("setting_control").height;
          text: "x"
          onClicked:
          {
              UM.ActiveTool.triggerAction("_cancelAddingConnector");
          }

      }

    }
  }

  }

  /******
  Window Five
  Adding ... (loader)
  *****/

  APProgressView {
    visible: UM.ActiveTool.properties.getValue("ActiveSubView") == 4
    id: windowFive
    labelText: "Adding... "
  }

  /****
  Window Four
  Creating... (loader)

  // in a separated view to use older styled progress bar

  ****/
  APProgressView {
    visible: UM.ActiveTool.properties.getValue("ActiveSubView") == 3
    id: windowFour
    labelText: "Creating... "
  }



}
