// Copyright (c) 2019 Ultimaker B.V.
// Uranium is released under the terms of the LGPLv3 or higher.

import QtQuick 2.2
import QtQuick.Controls 1.2

import Cura 1.0 as Cura
import UM 1.1 as UM

Item
{
    width: visibleChildren[0] ? visibleChildren[0].width : 0
    height: visibleChildren[0] ? visibleChildren[0].height : 0

    UM.I18nCatalog { id: catalog; name: "uranium"}

    Item {
      width: childrenRect.width
      height: childrenRect.height
      id: step2
      visible: UM.ActiveTool.properties.getValue("IsSlicing")

      Column
      {
          id: items2
          anchors.top: parent.top;
          anchors.left: parent.left;
          spacing: UM.Theme.getSize("default_margin").height;
          Text {
              id: cuttingLabel
              wrapMode: Text.WordWrap
              text: "Cutting... "
              font: UM.Theme.getFont("medium_bold")
              color: UM.Theme.getColor("text")
          }


          ProgressBar {
            indeterminate: true
          }



      }

    }

    Item {
      width: childrenRect.width
      height: childrenRect.height
      id: step1
      visible: !UM.ActiveTool.properties.getValue("IsSlicing")

      Column
      {
          id: items
          anchors.top: parent.top;
          anchors.left: parent.left;
          spacing: UM.Theme.getSize("default_margin").height;
          Text {
              id: titleLabel
              wrapMode: Text.WordWrap
              text: "Define cut surface "
              font: UM.Theme.getFont("medium_bold")
              color: UM.Theme.getColor("text")
          }

          Text {
              id: descriptionLabel
              width: 200
              text: "Move/Rotate the object on the buildplate to change the cut surface"
              wrapMode: Text.WordWrap
              font: UM.Theme.getFont("default")
              color: UM.Theme.getColor("text")

          }

          Row {
            width: items.width
            layoutDirection: Qt.RightToLeft
            spacing: UM.Theme.getSize("default_margin").height;


            Cura.PrimaryButton
            {
                id: selectPlaneButton;
                height: UM.Theme.getSize("setting_control").height;

                text: "Select"
                onClicked:
                {
                    UM.ActiveTool.triggerAction("_selectPlanePressed");
                }
            }

            Cura.SecondaryButton
            {

                id: cancelButton;
                height: UM.Theme.getSize("setting_control").height;

                text: "x"
                onClicked:
                {
                    UM.ActiveTool.triggerAction("_cancelPressed");
                }

            }


          }

          Text {
              visible: UM.ActiveTool.properties.getValue("ShowError")
              id: errorLabel
              width: 200
              text: UM.ActiveTool.properties.getValue("ErrorMessage")
              wrapMode: Text.WordWrap
              font: UM.Theme.getFont("default")
              color: UM.Theme.getColor("error")

          }

      }

    }






}
